<?php

namespace idoit\Console\Command\Cleanup;

use idoit\Console\Command\AbstractCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Formatter\OutputFormatterStyle;
use Symfony\Component\Console\Helper\Table;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\ChoiceQuestion;

class CleanupDuplicateSerialCommand extends AbstractCommand
{
    const NAME = 'cleanup-duplicate-serial';

    /**
     * @var OutputInterface
     */
    private $output;

    /**
     * @var InputInterface
     */
    private $input;

    /**
     * Get name for command
     *
     * @return string
     */
    public function getCommandName()
    {
        return self::NAME;
    }

    /**
     * Get description for command
     *
     * @return string
     */
    public function getCommandDescription()
    {
        return 'Removes duplicate serial numbers and keep the last serial number which has been updated.';
    }

    /**
     * Retrieve Command InputDefinition
     *
     * @return InputDefinition
     */
    public function getCommandDefinition()
    {
        $definition = new InputDefinition();

        $definition->addOption(new InputOption('execute', '-e', InputOption::VALUE_NONE, 'Executes all Delete Statements'));
        $definition->addOption(new InputOption('size', '-s', InputOption::VALUE_REQUIRED, 'Chunk size of viewable entries in the manuel cleanup procedure', 20));

        return $definition;
    }

    /**
     * Checks if a command can have a config file via --config
     *
     * @return bool
     */
    public function isConfigurable()
    {
        return true;
    }

    /**
     * Returns an array of command usages
     *
     * @return string[]
     */
    public function getCommandUsages()
    {
        return [];
    }

    /**
     * @param InputInterface  $input
     * @param OutputInterface $output
     *
     * @return int
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $this->output = $output;
        $this->input = $input;

        try {
            $outputStyle = new OutputFormatterStyle('yellow', '', ['bold']);
            $this->output->getFormatter()->setStyle('yellow', $outputStyle);
            $outputStyle = new OutputFormatterStyle('red', '', ['bold', 'blink']);
            $this->output->getFormatter()->setStyle('blink-red', $outputStyle);
            $outputStyle = new OutputFormatterStyle('yellow', '', ['bold', 'blink']);
            $this->output->getFormatter()->setStyle('blink-yellow', $outputStyle);
            $outputStyle = new OutputFormatterStyle('green', '', ['bold', 'blink']);
            $this->output->getFormatter()->setStyle('blink-green', $outputStyle);

            if ($input->getOption('execute') === false) {
                $this->output->writeln("<blink-yellow>The option for --execute has not been set. No datasets will be updated.</blink-yellow>");
            }

            if ($input->getOption('no-interaction')) {
                $this->automaticCleanup();
                return Command::SUCCESS;
            }

            $this->output->writeln('<info>Here a list of objects which have a duplicate serial number</info>:', true);
            $this->cleanupDuplicateSerial();
            return Command::SUCCESS;
        } catch (\Exception $exception) {
            $this->output->writeln('<error>' . $exception->getMessage() . '</error>');
        }

        $this->output->writeln('<error>Aborting cleanup</error>');
        return Command::FAILURE;
    }

    /**
     * @return array
     * @throws \Exception
     */
    private function getAllObjectsWithDuplicateSerial()
    {
        $database = $this->container->get('database');

        $query = "Select isys_catg_model_list__serial as 'serial'
            FROM isys_catg_model_list
            inner join isys_obj on isys_obj__id = isys_catg_model_list__isys_obj__id 
            where isys_catg_model_list__serial != ''
            group by isys_catg_model_list__serial having count(isys_catg_model_list__id) > 1";

        $objectQuery = 'SELECT isys_obj__id as id, isys_obj__title as title, isys_obj__updated as lastUp
            FROM isys_obj
            INNER JOIN isys_catg_model_list ON isys_obj__id = isys_catg_model_list__isys_obj__id
            WHERE isys_catg_model_list__serial = %s
            ORDER BY isys_obj__id DESC';
        $dao = $this->container->get('cmdb_dao');
        $result = $database->query($query);
        $data = [];
        $objectFormat = '%s| SERIAL: %s| TITLE: %s| LAST_UPDATED: %s';
        while ($row = $database->fetch_row_assoc($result)) {
            $serial = $row['serial'];
            $newQuery = sprintf($objectQuery, $dao->convert_sql_text($serial));

            $objectsResult = $database->query($newQuery);
            while ($objectData = $database->fetch_row_assoc($objectsResult)) {
                $data[] = sprintf($objectFormat, $objectData['id'], $serial, $objectData['title'], $objectData['lastUp']);
            }
        }
        return $data;
    }

    /**
     * @throws \Exception
     */
    private function automaticCleanup()
    {
        $this->output->writeln('<info>Starting automatic cleanup.</info>');

        $duplicateSerials = $this->getAllObjectsWithDuplicateSerial();
        $remove = $keep = [];

        $update = 'UPDATE isys_catg_model_list SET isys_catg_model_list__serial = NULL
            WHERE isys_catg_model_list__isys_obj__id IN (%s)';

        foreach ($duplicateSerials as $objectRow) {
            $objectRawData = explode('|', trim($objectRow));
            $objectId = (int)$objectRawData[0];
            $objectTitle = trim($objectRawData[2]);
            $objectTitle = trim(substr($objectTitle, strpos($objectTitle, ':') + 1));
            $serial = trim($objectRawData[1]);
            $serial = trim(substr($serial, strpos($serial, ':') + 1));
            $lastUpdated = trim($objectRawData[3]);
            $lastUpdated = trim(substr($lastUpdated, strpos($lastUpdated, ':') + 1));
            $data = [
                'id' => $objectId,
                'title' => $objectTitle,
                'updated' => $lastUpdated
            ];

            if (isset($keep[$serial])) {
                if ($keep[$serial]['updated'] > $lastUpdated) {
                    $remove[] = $objectId;
                    $this->output->writeln("<yellow>ID: {$objectId}| Object: {$objectTitle}| Serial: {$serial}</yellow> <blink-red>Removed</blink-red>");
                    continue;
                }
                if ($keep[$serial]['updated'] < $lastUpdated) {
                    $remove[] = $keep[$serial]['id'];
                    $this->output->writeln("<yellow>ID: {$keep[$serial]['id']}| Object: {$keep[$serial]['title']}| Serial: {$serial}</yellow> <blink-red>Removed</blink-red>");
                }
            }
            $keep[$serial] = $data;
        }

        if (count($remove) > 0) {
            $database = $this->container->get('database');
            if ($this->input->getOption('execute')) {
                $updateQuery = sprintf($update, implode(',', $remove));
                $database->query($updateQuery);
            }
        }

        if (count($keep) > 0) {
            $this->output->writeln("<info>Kept Serial numbers:</info>");
            foreach ($keep as $serial => $item) {
                $this->output->writeln("<yellow>ID: {$item['id']}| Object: {$item['title']}| Serial: {$serial}</yellow> <blink-green>Kept!</blink-green>");
            }
        }
    }

    /**
     * @param string $question
     * @param array  $options
     *
     * @return mixed|string|null
     */
    private function buildQuestion(string $question, array $options)
    {
        $questionAutomatic = new ChoiceQuestion(
            $question,
            $options
        );
        return $this->getHelper('question')->ask($this->input, $this->output, $questionAutomatic);
    }

    /**
     * @throws \Exception
     */
    private function renderDuplicatedSerial()
    {
        $database = $this->container->get('database');

        $query = "Select isys_catg_model_list__serial as 'serial', 
            group_concat(isys_obj__title) as 'objects',  
            count(isys_catg_model_list__id) as 'duplications'
        from isys_catg_model_list
            inner join isys_obj on isys_obj__id = isys_catg_model_list__isys_obj__id 
            where isys_catg_model_list__serial != ''
            group by isys_catg_model_list__serial having duplications > 1";

        $result = $database->query($query);

        if ($database->num_rows($result) === 0) {
            $this->output->writeln('<comment>No duplicate serials found.</comment>', true);
            return;
        }
        $table = new Table($this->output);
        $table->setHeaders(['Serial number', 'Objects', 'Duplications']);
        $data = [];
        while ($row = $database->fetch_row_assoc($result)) {
            $data[] = [$row['serial'], $row['objects'], $row['duplications']];
        }
        $table->setRows($data);
        $table->render();
    }

    /**
     * @throws \Exception
     */
    private function cleanupDuplicateSerial()
    {
        $this->renderDuplicatedSerial();

        $helper = $this->getHelper('question');

        if ($this->buildQuestion('Do you wish an automatic cleanup?', ['Yes', 'No']) === 'Yes') {
            $this->automaticCleanup();
            return;
        }

        if ($this->buildQuestion('Do you want a manual cleanup?', ['Yes', 'No']) === 'No') {
            $this->output->writeln('<error>Aborting cleanup process.</error>');
            return;
        }

        $duplicateSerials = $this->getAllObjectsWithDuplicateSerial();

        $chunks = array_chunk($duplicateSerials, $this->input->getOption('size'));
        $serialsRemoved = $updateObjects = [];

        $update = 'UPDATE isys_catg_model_list 
            SET isys_catg_model_list__serial = NULL
            WHERE isys_catg_model_list__isys_obj__id IN (%s);';

        foreach ($chunks as $dupes) {
            $question = new ChoiceQuestion('Please select the objects you want to cleanup.', $dupes);
            $question
                ->setMultiselect(true)
                ->setMaxAttempts(2);

            $test = $helper->ask($this->input, $this->output, $question);

            foreach ($test as $objectLine) {
                $objectRawData = explode('|', trim($objectLine));
                $objectId = (int)$objectRawData[0];
                $objectTitle = trim($objectRawData[2]);
                $serial = trim($objectRawData[1]);

                if ($objectId > 0) {
                    $updateObjects[] = $objectId;
                    $serialsRemoved[] = $objectId . '|' . $objectTitle . '|' . $serial;
                }
            }
        }

        if (count($updateObjects) > 0) {
            if ($this->input->getOption('execute')) {
                $database->query(sprintf($update, implode(',', $updateObjects)));
            }
            $this->output->writeln("<info>Serials from following objects have been unset:</info>");
            foreach ($serialsRemoved as $line) {
                $this->output->writeln("<comment>{$line}</comment>");
            }
        } else {
            $this->output->writeln("<error>No objects have been selected.</error>");
        }
    }
}
